#!/usr/bin/env python3
"""
PPTX 文件自定义属性读取工具
"""

import os
import sys
import zipfile
import xml.etree.ElementTree as ET
from typing import Dict, Any, Optional
import shutil


def read_pptx_properties(filepath: str) -> Dict[str, Any]:
    """
    读取 PPTX 文件的自定义属性
    
    Args:
        filepath: PPTX 文件路径
        
    Returns:
        包含文件属性的字典
    """
    if not os.path.exists(filepath):
        raise FileNotFoundError(f"文件不存在: {filepath}")
    
    if not filepath.lower().endswith('.pptx'):
        raise ValueError(f"文件不是 PPTX 格式: {filepath}")
    
    properties = {
        'core_properties': {},
        'app_properties': {},
        'custom_properties': {}
    }
    
    try:
        with zipfile.ZipFile(filepath, 'r') as pptx_zip:
            # 读取核心属性 (core.xml)
            if 'docProps/core.xml' in pptx_zip.namelist():
                core_xml = pptx_zip.read('docProps/core.xml')
                properties['core_properties'] = parse_core_properties(core_xml)
            
            # 读取应用属性 (app.xml)
            if 'docProps/app.xml' in pptx_zip.namelist():
                app_xml = pptx_zip.read('docProps/app.xml')
                properties['app_properties'] = parse_app_properties(app_xml)
            
            # 读取自定义属性 (custom.xml)
            if 'docProps/custom.xml' in pptx_zip.namelist():
                custom_xml = pptx_zip.read('docProps/custom.xml')
                properties['custom_properties'] = parse_custom_properties(custom_xml)
    
    except zipfile.BadZipFile:
        raise ValueError(f"文件不是有效的 PPTX 格式: {filepath}")
    except Exception as e:
        raise RuntimeError(f"读取文件时发生错误: {e}")
    
    return properties


def parse_core_properties(xml_content: bytes) -> Dict[str, str]:
    """解析核心属性 XML"""
    properties = {}
    
    # 定义命名空间
    namespaces = {
        'cp': 'http://schemas.openxmlformats.org/package/2006/metadata/core-properties',
        'dc': 'http://purl.org/dc/elements/1.1/',
        'dcterms': 'http://purl.org/dc/terms/',
        'dcmitype': 'http://purl.org/dc/dcmitype/',
        'xsi': 'http://www.w3.org/2001/XMLSchema-instance'
    }
    
    try:
        root = ET.fromstring(xml_content)
        
        # 提取常用属性
        property_mappings = {
            'title': './/dc:title',
            'subject': './/dc:subject',
            'creator': './/dc:creator',
            'keywords': './/cp:keywords',
            'description': './/dc:description',
            'language': './/dc:language',
            'category': './/cp:category',
            'version': './/cp:version',
            'revision': './/cp:revision',
            'lastModifiedBy': './/cp:lastModifiedBy',
            'created': './/dcterms:created',
            'modified': './/dcterms:modified'
        }
        
        for prop_name, xpath in property_mappings.items():
            element = root.find(xpath, namespaces)
            if element is not None and element.text:
                properties[prop_name] = element.text.strip()
    
    except ET.ParseError as e:
        print(f"解析核心属性 XML 时出错: {e}", file=sys.stderr)
    
    return properties


def parse_app_properties(xml_content: bytes) -> Dict[str, str]:
    """解析应用属性 XML"""
    properties = {}
    
    # 定义命名空间
    namespaces = {
        'ep': 'http://schemas.openxmlformats.org/officeDocument/2006/extended-properties'
    }
    
    try:
        root = ET.fromstring(xml_content)
        
        # 提取常用属性
        property_mappings = {
            'application': './/ep:Application',
            'docSecurity': './/ep:DocSecurity',
            'scaleCrop': './/ep:ScaleCrop',
            'linksUpToDate': './/ep:LinksUpToDate',
            'pages': './/ep:Pages',
            'words': './/ep:Words',
            'characters': './/ep:Characters',
            'presentationFormat': './/ep:PresentationFormat',
            'paragraphs': './/ep:Paragraphs',
            'slides': './/ep:Slides',
            'notes': './/ep:Notes',
            'totalTime': './/ep:TotalTime',
            'hiddenSlides': './/ep:HiddenSlides',
            'mmClips': './/ep:MMClips',
            'headingPairs': './/ep:HeadingPairs',
            'titlesOfParts': './/ep:TitlesOfParts',
            'manager': './/ep:Manager',
            'company': './/ep:Company',
            'lines': './/ep:Lines',
            'paragraphs': './/ep:Paragraphs',
            'slides': './/ep:Slides',
            'notes': './/ep:Notes',
            'totalTime': './/ep:TotalTime',
            'hiddenSlides': './/ep:HiddenSlides',
            'mmClips': './/ep:MMClips'
        }
        
        for prop_name, xpath in property_mappings.items():
            element = root.find(xpath, namespaces)
            if element is not None and element.text:
                properties[prop_name] = element.text.strip()
    
    except ET.ParseError as e:
        print(f"解析应用属性 XML 时出错: {e}", file=sys.stderr)
    
    return properties


def parse_custom_properties(xml_content: bytes) -> Dict[str, str]:
    """解析自定义属性 XML"""
    properties = {}
    
    # 定义命名空间
    namespaces = {
        'cp': 'http://schemas.openxmlformats.org/officeDocument/2006/custom-properties',
        'vt': 'http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes'
    }
    
    try:
        root = ET.fromstring(xml_content)
        
        # 查找所有自定义属性 - 使用正确的命名空间
        for prop in root.findall('.//cp:property', namespaces):
            name = prop.get('name')
            if name:
                # 查找属性值
                value_elem = prop.find('.//vt:lpwstr', namespaces)
                if value_elem is not None and value_elem.text:
                    properties[name] = value_elem.text.strip()
                else:
                    # 尝试其他数据类型
                    for vt_type in ['vt:i4', 'vt:r8', 'vt:bool', 'vt:filetime']:
                        value_elem = prop.find(f'.//{vt_type}', namespaces)
                        if value_elem is not None and value_elem.text:
                            properties[name] = value_elem.text.strip()
                            break
    
    except ET.ParseError as e:
        print(f"解析自定义属性 XML 时出错: {e}", file=sys.stderr)
    
    return properties


def format_properties(properties: Dict[str, Any]) -> str:
    """格式化属性输出"""
    output = []
    
    # 核心属性
    if properties['core_properties']:
        output.append("=== 核心属性 ===")
        for key, value in properties['core_properties'].items():
            output.append(f"  {key}: {value}")
        output.append("")
    
    # 应用属性
    if properties['app_properties']:
        output.append("=== 应用属性 ===")
        for key, value in properties['app_properties'].items():
            output.append(f"  {key}: {value}")
        output.append("")
    
    # 自定义属性
    if properties['custom_properties']:
        output.append("=== 自定义属性 ===")
        for key, value in properties['custom_properties'].items():
            output.append(f"  {key}: {value}")
        output.append("")
    
    if not any(properties.values()):
        output.append("未找到任何属性信息")
    
    return "\n".join(output)


def clear_pptx_properties(filepath: str, properties_to_clear: list) -> None:
    """
    清除 PPTX 文件中的指定属性
    
    Args:
        filepath: PPTX 文件路径
        properties_to_clear: 要清除的属性列表，格式为 ['core:lastModifiedBy', 'custom:ICV']
    """
    if not os.path.exists(filepath):
        raise FileNotFoundError(f"文件不存在: {filepath}")
    
    if not filepath.lower().endswith('.pptx'):
        raise ValueError(f"文件不是 PPTX 格式: {filepath}")
    
    try:
        with zipfile.ZipFile(filepath, 'r') as pptx_zip:
            # 创建临时文件列表
            temp_files = {}
            
            # 处理核心属性
            if 'docProps/core.xml' in pptx_zip.namelist():
                core_xml = pptx_zip.read('docProps/core.xml')
                modified_core = clear_core_property(core_xml, properties_to_clear)
                if modified_core:
                    temp_files['docProps/core.xml'] = modified_core
            
            # 处理自定义属性
            if 'docProps/custom.xml' in pptx_zip.namelist():
                custom_xml = pptx_zip.read('docProps/custom.xml')
                modified_custom = clear_custom_property(custom_xml, properties_to_clear)
                if modified_custom:
                    temp_files['docProps/custom.xml'] = modified_custom
            
            # 如果有修改，创建新的 PPTX 文件
            if temp_files:
                # 创建临时文件
                temp_pptx = filepath + '.tmp'
                
                try:
                    # 创建全新的 ZIP 文件，保持原有的压缩算法
                    with zipfile.ZipFile(filepath, 'r') as original_zip:
                        # 获取原文件的压缩算法
                        compression = zipfile.ZIP_DEFLATED
                        # 检查第一个文件的压缩方法作为参考
                        if original_zip.infolist():
                            compression = original_zip.infolist()[0].compress_type
                        
                        with zipfile.ZipFile(temp_pptx, 'w', compression=compression) as new_zip:
                            # 复制所有原始文件
                            for item in original_zip.infolist():
                                if item.filename not in temp_files:
                                    # 复制未修改的文件，保持原有的压缩级别
                                    new_zip.writestr(item.filename, original_zip.read(item.filename), 
                                                   compress_type=item.compress_type)
                                else:
                                    # 写入修改后的文件，使用相同的压缩算法
                                    new_zip.writestr(item.filename, temp_files[item.filename], 
                                                   compress_type=compression)
                    
                    # 替换原文件
                    shutil.move(temp_pptx, filepath)
                    print(f"已成功清除属性并保存到文件: {filepath}")
                    
                except Exception as e:
                    # 清理临时文件
                    if os.path.exists(temp_pptx):
                        os.remove(temp_pptx)
                    raise e
            else:
                print("未找到需要清除的属性")
    
    except zipfile.BadZipFile:
        raise ValueError(f"文件不是有效的 PPTX 格式: {filepath}")
    except Exception as e:
        raise RuntimeError(f"清除属性时发生错误: {e}")


def clear_core_property(xml_content: bytes, properties_to_clear: list) -> Optional[bytes]:
    """清除核心属性中的指定属性"""
    namespaces = {
        'cp': 'http://schemas.openxmlformats.org/package/2006/metadata/core-properties',
        'dc': 'http://purl.org/dc/elements/1.1/',
        'dcterms': 'http://purl.org/dc/terms/',
        'dcmitype': 'http://purl.org/dc/dcmitype/',
        'xsi': 'http://www.w3.org/2001/XMLSchema-instance'
    }
    
    try:
        root = ET.fromstring(xml_content)
        modified = False
        
        for prop in properties_to_clear:
            if prop.startswith('core:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'lastModifiedBy':
                    # 查找并移除 lastModifiedBy 元素
                    element = root.find('.//cp:lastModifiedBy', namespaces)
                    if element is not None:
                        parent = root.find('.//cp:lastModifiedBy/..', namespaces)
                        if parent is not None:
                            parent.remove(element)
                            modified = True
        
        if modified:
            return ET.tostring(root, encoding='utf-8', xml_declaration=True)
        return None
    
    except ET.ParseError as e:
        print(f"解析核心属性 XML 时出错: {e}", file=sys.stderr)
        return None


def clear_custom_property(xml_content: bytes, properties_to_clear: list) -> Optional[bytes]:
    """清除自定义属性中的指定属性"""
    namespaces = {
        'cp': 'http://schemas.openxmlformats.org/officeDocument/2006/custom-properties',
        'vt': 'http://schemas.openxmlformats.org/officeDocument/2006/docPropsVTypes'
    }
    
    try:
        root = ET.fromstring(xml_content)
        modified = False
        
        for prop in properties_to_clear:
            if prop.startswith('custom:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'ICV':
                    # 查找并移除 ICV 属性
                    # 使用与 parse_custom_properties 相同的方法来查找属性
                    for prop_elem in root.findall('.//cp:property', namespaces):
                        name_attr = prop_elem.get('name')
                        if name_attr == 'ICV':
                            # 找到父元素并移除该属性
                            for parent in root.iter():
                                if prop_elem in list(parent):
                                    parent.remove(prop_elem)
                                    modified = True
                                    break
        
        if modified:
            return ET.tostring(root, encoding='utf-8', xml_declaration=True)
        return None
    
    except ET.ParseError as e:
        print(f"解析自定义属性 XML 时出错: {e}", file=sys.stderr)
        return None


def command_pptattr(filepath: str, clear_properties: bool = False) -> None:
    """
    执行 pptattr 命令
    
    Args:
        filepath: PPTX 文件路径
        clear_properties: 是否清除指定属性
    """
    try:
        if clear_properties:
            # 清除指定的属性
            properties_to_clear = ['core:lastModifiedBy', 'custom:ICV']
            # 优先使用保持 XML 结构的方法
            clear_pptx_properties_preserve_xml(filepath, properties_to_clear)
            
            print(f"清除后属性如下：")            
            properties = read_pptx_properties(filepath)
            print(format_properties(properties))
        else:
            # 读取并显示属性
            properties = read_pptx_properties(filepath)
            print(f"文件: {filepath}")
            print(format_properties(properties))
    
    except FileNotFoundError as e:
        print(f"错误: {e}", file=sys.stderr)
        sys.exit(1)
    except ValueError as e:
        print(f"错误: {e}", file=sys.stderr)
        sys.exit(1)
    except RuntimeError as e:
        print(f"错误: {e}", file=sys.stderr)
        sys.exit(1)
    except Exception as e:
        print(f"未知错误: {e}", file=sys.stderr)
        sys.exit(1) 


def clear_pptx_properties_with_docx(filepath: str, properties_to_clear: list) -> None:
    """
    使用 python-docx 库清除 PPTX 文件中的指定属性（保持原有 XML 结构）
    
    Args:
        filepath: PPTX 文件路径
        properties_to_clear: 要清除的属性列表，格式为 ['core:lastModifiedBy', 'custom:ICV']
    """
    try:
        from docx import Document
        from docx.opc.constants import CONTENT_TYPE as CT
        from docx.oxml import parse_xml
        from docx.oxml.ns import qn
    except ImportError:
        print("警告: 未安装 python-docx 库，将使用备用方法")
        print("建议安装: pip install python-docx")
        return clear_pptx_properties(filepath, properties_to_clear)
    
    if not os.path.exists(filepath):
        raise FileNotFoundError(f"文件不存在: {filepath}")
    
    if not filepath.lower().endswith('.pptx'):
        raise ValueError(f"文件不是 PPTX 格式: {filepath}")
    
    try:
        # 使用 python-docx 打开文档
        doc = Document(filepath)
        
        # 获取核心属性
        core_props = doc.core_properties
        custom_props = doc.custom_properties
        
        modified = False
        
        # 清除核心属性
        for prop in properties_to_clear:
            if prop.startswith('core:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'lastModifiedBy' and hasattr(core_props, 'last_modified_by'):
                    if core_props.last_modified_by:
                        core_props.last_modified_by = None
                        modified = True
        
        # 清除自定义属性
        for prop in properties_to_clear:
            if prop.startswith('custom:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'ICV' and prop_name in custom_props:
                    del custom_props[prop_name]
                    modified = True
        
        if modified:
            # 保存文档
            doc.save(filepath)
            print(f"已成功清除属性并保存到文件: {filepath}")
        else:
            print("未找到需要清除的属性")
    
    except Exception as e:
        print(f"使用 python-docx 清除属性时发生错误: {e}")
        print("将使用备用方法...")
        return clear_pptx_properties(filepath, properties_to_clear)


def clear_pptx_properties_preserve_xml(filepath: str, properties_to_clear: list) -> None:
    """
    清除 PPTX 文件中的指定属性，尽可能保持原有 XML 结构
    
    Args:
        filepath: PPTX 文件路径
        properties_to_clear: 要清除的属性列表，格式为 ['core:lastModifiedBy', 'custom:ICV']
    """
    if not os.path.exists(filepath):
        raise FileNotFoundError(f"文件不存在: {filepath}")
    
    if not filepath.lower().endswith('.pptx'):
        raise ValueError(f"文件不是 PPTX 格式: {filepath}")
    
    try:
        with zipfile.ZipFile(filepath, 'r') as pptx_zip:
            # 创建临时文件列表
            temp_files = {}
            
            # 处理核心属性
            if 'docProps/core.xml' in pptx_zip.namelist():
                core_xml = pptx_zip.read('docProps/core.xml')
                modified_core = clear_core_property_preserve_xml(core_xml, properties_to_clear)
                if modified_core:
                    temp_files['docProps/core.xml'] = modified_core
            
            # 处理自定义属性
            if 'docProps/custom.xml' in pptx_zip.namelist():
                custom_xml = pptx_zip.read('docProps/custom.xml')
                modified_custom = clear_custom_property_preserve_xml(custom_xml, properties_to_clear)
                if modified_custom:
                    temp_files['docProps/custom.xml'] = modified_custom
            
            # 如果有修改，创建新的 PPTX 文件
            if temp_files:
                # 创建临时文件
                temp_pptx = filepath + '.tmp'
                shutil.copy2(filepath, temp_pptx)
                
                try:
                    # 创建全新的 ZIP 文件，保持原有的压缩算法
                    with zipfile.ZipFile(filepath, 'r') as original_zip:
                        # 获取原文件的压缩算法
                        compression = zipfile.ZIP_DEFLATED
                        # 检查第一个文件的压缩方法作为参考
                        if original_zip.infolist():
                            compression = original_zip.infolist()[0].compress_type
                        
                        with zipfile.ZipFile(temp_pptx, 'w', compression=compression) as new_zip:
                            # 复制所有原始文件
                            for item in original_zip.infolist():
                                if item.filename not in temp_files:
                                    # 复制未修改的文件，保持原有的压缩级别
                                    new_zip.writestr(item.filename, original_zip.read(item.filename), 
                                                   compress_type=item.compress_type)
                                else:
                                    # 写入修改后的文件，使用相同的压缩算法
                                    new_zip.writestr(item.filename, temp_files[item.filename], 
                                                   compress_type=compression)
                    
                    # 替换原文件
                    shutil.move(temp_pptx, filepath)
                    print(f"已成功清除属性并保存到文件: {filepath}")
                    
                except Exception as e:
                    # 清理临时文件
                    if os.path.exists(temp_pptx):
                        os.remove(temp_pptx)
                    raise e
            else:
                print("未找到需要清除的属性")
    
    except zipfile.BadZipFile:
        raise ValueError(f"文件不是有效的 PPTX 格式: {filepath}")
    except Exception as e:
        raise RuntimeError(f"清除属性时发生错误: {e}")


def clear_core_property_preserve_xml(xml_content: bytes, properties_to_clear: list) -> Optional[bytes]:
    """清除核心属性中的指定属性，保持原有 XML 结构"""
    try:
        # 使用字符串操作来保持原有的 XML 结构
        xml_str = xml_content.decode('utf-8')
        modified = False
        
        for prop in properties_to_clear:
            if prop.startswith('core:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'lastModifiedBy':
                    # 查找并移除 lastModifiedBy 标签及其内容
                    import re
                    pattern = r'<cp:lastModifiedBy>[^<]*</cp:lastModifiedBy>'
                    if re.search(pattern, xml_str):
                        xml_str = re.sub(pattern, '', xml_str)
                        modified = True
        
        if modified:
            return xml_str.encode('utf-8')
        return None
    
    except Exception as e:
        print(f"清除核心属性时出错: {e}", file=sys.stderr)
        return None


def clear_custom_property_preserve_xml(xml_content: bytes, properties_to_clear: list) -> Optional[bytes]:
    """清除自定义属性中的指定属性，保持原有 XML 结构"""
    try:
        # 使用字符串操作来保持原有的 XML 结构
        xml_str = xml_content.decode('utf-8')
        modified = False
        
        for prop in properties_to_clear:
            if prop.startswith('custom:'):
                prop_name = prop.split(':', 1)[1]
                if prop_name == 'ICV':
                    # 查找并移除 ICV 属性标签及其内容
                    import re
                    # 匹配完整的 property 标签，包括 name="ICV" 的属性
                    pattern = r'<property[^>]*name="ICV"[^>]*>.*?</property>'
                    if re.search(pattern, xml_str, re.DOTALL):
                        xml_str = re.sub(pattern, '', xml_str, flags=re.DOTALL)
                        modified = True
        
        if modified:
            return xml_str.encode('utf-8')
        return None
    
    except Exception as e:
        print(f"清除自定义属性时出错: {e}", file=sys.stderr)
        return None 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
列出所有端口使用情况（跨平台）。

优先使用 psutil（若安装），否则基于系统自带工具：
- Windows: netstat + tasklist + wmic
- Linux: ss（优先）/netstat + ps
- macOS: lsof + ps
"""

from __future__ import annotations

import platform
import re
import shutil
import subprocess
from typing import List, Tuple, Optional, Dict


def _run_command(command_args: List[str]) -> Tuple[int, str]:
    """运行命令并返回 (returncode, stdout)"""
    process = subprocess.Popen(
        command_args,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        encoding='utf-8',
        errors='replace',
    )
    assert process.stdout is not None
    output_lines = []
    for line in process.stdout:
        output_lines.append(line)
    process.wait()
    return process.returncode, ''.join(output_lines)


def _get_all_process_info_windows() -> Dict[int, Tuple[str, str]]:
    """批量获取所有 Windows 进程信息（进程名和命令行）
    
    返回: Dict[pid, (name, cmdline)]
    """
    pid_info: Dict[int, Tuple[str, str]] = {}
    
    # 方法1: 尝试使用 wmic 批量获取（最快）
    try:
        rc, out = _run_command(["wmic", "process", "get", "ProcessId,Name,CommandLine", "/format:csv"])
        if rc == 0:
            lines = out.splitlines()
            for line in lines[1:]:  # 跳过表头
                if not line.strip():
                    continue
                # CSV 格式: Node,ProcessId,Name,CommandLine
                parts = line.split(',')
                if len(parts) >= 4:
                    try:
                        pid = int(parts[1])
                        name = parts[2].strip()
                        cmdline = ','.join(parts[3:]).strip()  # CommandLine 可能包含逗号
                        if pid > 0:
                            pid_info[pid] = (name, cmdline)
                    except (ValueError, IndexError):
                        continue
            if pid_info:
                return pid_info
    except Exception:
        pass
    
    # 方法2: 使用 PowerShell 批量获取（备用方案）
    try:
        ps_cmd = "Get-WmiObject Win32_Process | Select-Object ProcessId,Name,CommandLine | ConvertTo-Csv -NoTypeInformation"
        rc, out = _run_command(["powershell", "-Command", ps_cmd])
        if rc == 0:
            lines = out.splitlines()
            for line in lines[1:]:  # 跳过表头
                if not line.strip():
                    continue
                # CSV 格式: "ProcessId","Name","CommandLine"
                # 使用正则表达式解析带引号的 CSV
                match = re.match(r'"(\d+)","([^"]*)","(.*)"', line)
                if match:
                    try:
                        pid = int(match.group(1))
                        name = match.group(2)
                        cmdline = match.group(3)
                        if pid > 0:
                            pid_info[pid] = (name, cmdline)
                    except (ValueError, IndexError):
                        continue
            if pid_info:
                return pid_info
    except Exception:
        pass
    
    # 方法3: 使用 tasklist 获取进程名（至少能获取进程名）
    try:
        rc, out = _run_command(["tasklist", "/FO", "CSV", "/NH"])
        if rc == 0:
            for line in out.splitlines():
                if not line.strip():
                    continue
                # CSV 格式: "Image Name","PID","Session Name","Session#","Mem Usage"
                parts = line.split('","')
                if len(parts) >= 2:
                    try:
                        name = parts[0].strip('"')
                        pid = int(parts[1].strip('"'))
                        if pid > 0:
                            # 如果没有命令行信息，至少保存进程名
                            if pid not in pid_info:
                                pid_info[pid] = (name, "")
                            else:
                                # 更新进程名
                                old_name, old_cmdline = pid_info[pid]
                                pid_info[pid] = (name, old_cmdline)
                    except (ValueError, IndexError):
                        continue
    except Exception:
        pass
    
    return pid_info


def _get_all_process_info_linux(pids: set) -> Dict[int, str]:
    """批量获取 Linux 进程的命令行
    
    返回: Dict[pid, cmdline]
    """
    pid_cmdline: Dict[int, str] = {}
    for pid in pids:
        try:
            # 读取 /proc/<pid>/cmdline
            with open(f"/proc/{pid}/cmdline", "r", encoding='utf-8', errors='replace') as f:
                cmdline = f.read().replace('\0', ' ').strip()
                if cmdline:
                    pid_cmdline[pid] = cmdline
        except Exception:
            pass
    return pid_cmdline


def _get_all_process_info_mac() -> Dict[int, str]:
    """批量获取所有 macOS 进程的命令行
    
    返回: Dict[pid, cmdline]
    """
    pid_cmdline: Dict[int, str] = {}
    try:
        # 一次性获取所有进程的命令行
        rc, out = _run_command(["ps", "-eo", "pid,command"])
        if rc == 0:
            for line in out.splitlines():
                if not line.strip():
                    continue
                # 格式: PID COMMAND
                parts = line.split(None, 1)  # 只分割第一个空格
                if len(parts) >= 2:
                    try:
                        pid = int(parts[0])
                        cmdline = parts[1].strip()
                        if pid > 0:
                            pid_cmdline[pid] = cmdline
                    except (ValueError, IndexError):
                        continue
    except Exception:
        pass
    return pid_cmdline


def _try_psutil() -> List[Tuple[int, int, str, str, str]]:
    """使用 psutil 获取所有端口使用情况
    
    返回: List[(port, pid, name, status, cmdline)]
    """
    try:
        import psutil  # type: ignore
    except Exception:
        return []

    results: List[Tuple[int, int, str, str, str]] = []
    seen = set()  # (port, pid) 去重
    
    try:
        for conn in psutil.net_connections(kind='inet'):
            laddr = conn.laddr if conn.laddr else None
            if not laddr:
                continue
            
            port = getattr(laddr, 'port', None)
            if port is None:
                continue
            
            pid = conn.pid
            if pid is None:
                continue
            
            # 去重
            key = (port, pid)
            if key in seen:
                continue
            seen.add(key)
            
            # 获取进程信息
            try:
                proc = psutil.Process(pid)
                name = proc.name()
                try:
                    cmdline = ' '.join(proc.cmdline())
                except Exception:
                    cmdline = ""
                status = conn.status if hasattr(conn, 'status') else ""
            except Exception:
                name = "<unknown>"
                cmdline = ""
                status = ""
            
            results.append((port, pid, name, status, cmdline))
    except Exception:
        return []
    
    return results


def _windows_list() -> List[Tuple[int, int, str, str, str]]:
    """Windows 系统列出所有端口使用情况
    
    返回: List[(port, pid, name, status, cmdline)]
    """
    results: List[Tuple[int, int, str, str, str]] = []
    seen = set()  # (port, pid) 去重
    
    # 获取所有端口连接
    rc, out = _run_command(["netstat", "-ano"])
    if rc != 0:
        return results
    
    # 解析 netstat 输出
    # 格式: TCP    0.0.0.0:8080    0.0.0.0:0    LISTENING    1234
    pattern = re.compile(r"^\s*(TCP|UDP)\s+[^\s]*:(\d+)\s+.*\s+(\d+)\s*$", re.IGNORECASE)
    
    port_pid_map: Dict[Tuple[int, int], str] = {}  # {(port, pid): status}
    
    for line in out.splitlines():
        m = pattern.match(line)
        if not m:
            continue
        protocol = m.group(1).upper()
        port = int(m.group(2))
        pid = int(m.group(3))
        
        # 提取状态（如果有）
        status = ""
        if "LISTENING" in line:
            status = "LISTEN"
        elif "ESTABLISHED" in line:
            status = "ESTABLISHED"
        elif "TIME_WAIT" in line:
            status = "TIME_WAIT"
        
        key = (port, pid)
        if key not in port_pid_map:
            port_pid_map[key] = status
    
    # 批量获取所有进程信息（优化：一次性获取，而不是逐个查询）
    all_pids = {pid for (port, pid), status in port_pid_map.items()}
    pid_info = _get_all_process_info_windows()
    
    # 构建结果
    for (port, pid), status in port_pid_map.items():
        if pid in pid_info:
            name, cmdline = pid_info[pid]
        else:
            name = "<unknown>"
            cmdline = ""
        results.append((port, pid, name, status, cmdline))
    
    return results


def _linux_list() -> List[Tuple[int, int, str, str, str]]:
    """Linux 系统列出所有端口使用情况
    
    返回: List[(port, pid, name, status, cmdline)]
    """
    results: List[Tuple[int, int, str, str, str]] = []
    seen = set()  # (port, pid) 去重
    
    # 优先使用 ss
    if shutil.which("ss") is not None:
        rc, out_tcp = _run_command(["ss", "-ltnp"])  # TCP
        rc2, out_udp = _run_command(["ss", "-lunp"])  # UDP
        combined = (out_tcp if rc == 0 else "") + "\n" + (out_udp if rc2 == 0 else "")
        
        # 解析 ss 输出
        # 示例: LISTEN 0 128 0.0.0.0:8080 0.0.0.0:* users:(("nginx",pid=1234,fd=6))
        for line in combined.splitlines():
            if not line.strip() or "State" in line:
                continue
            
            # 提取端口
            port_match = re.search(r":(\d+)(?:\s|$)", line)
            if not port_match:
                continue
            port = int(port_match.group(1))
            
            # 提取状态
            status = ""
            if "LISTEN" in line:
                status = "LISTEN"
            elif "ESTAB" in line:
                status = "ESTABLISHED"
            
            # 提取进程信息
            proc_match = re.search(r'users:\(\(\"([^\"]+)\",pid=(\d+)', line)
            if proc_match:
                name = proc_match.group(1)
                pid = int(proc_match.group(2))
                
                key = (port, pid)
                if key in seen:
                    continue
                seen.add(key)
                
                # 先收集所有 PID，稍后批量获取命令行
                results.append((port, pid, name, status, ""))  # 临时空命令行
        
        # 批量获取所有进程的命令行（优化：一次性获取）
        if results:
            all_pids = {pid for port, pid, name, status, cmdline in results}
            pid_cmdline_map = _get_all_process_info_linux(all_pids)
            
            # 更新结果中的命令行
            final_results = []
            for port, pid, name, status, _ in results:
                cmdline = pid_cmdline_map.get(pid, "")
                final_results.append((port, pid, name, status, cmdline))
            return final_results
    
    # 回退使用 netstat
    if shutil.which("netstat") is not None:
        rc, out = _run_command(["netstat", "-tunlp"])
        if rc == 0:
            for line in out.splitlines():
                if not line.strip() or "Proto" in line:
                    continue
                
                # 提取端口和进程
                # 示例: tcp  0  0  0.0.0.0:8080  0.0.0.0:*  LISTEN  1234/nginx
                port_match = re.search(r":(\d+)(?:\s|$)", line)
                if not port_match:
                    continue
                port = int(port_match.group(1))
                
                proc_match = re.search(r"\s(\d+)/(\S+)\s*$", line)
                if not proc_match:
                    continue
                
                pid = int(proc_match.group(1))
                name = proc_match.group(2)
                
                key = (port, pid)
                if key in seen:
                    continue
                seen.add(key)
                
                # 提取状态
                status = ""
                if "LISTEN" in line:
                    status = "LISTEN"
                elif "ESTABLISHED" in line:
                    status = "ESTABLISHED"
                
                # 先收集所有 PID，稍后批量获取命令行
                results.append((port, pid, name, status, ""))  # 临时空命令行
    
    # 批量获取所有进程的命令行（优化：一次性获取）
    if results:
        all_pids = {pid for port, pid, name, status, cmdline in results}
        pid_cmdline_map = _get_all_process_info_linux(all_pids)
        
        # 更新结果中的命令行
        final_results = []
        for port, pid, name, status, _ in results:
            cmdline = pid_cmdline_map.get(pid, "")
            final_results.append((port, pid, name, status, cmdline))
        return final_results
    
    return results


def _mac_list() -> List[Tuple[int, int, str, str, str]]:
    """macOS 系统列出所有端口使用情况
    
    返回: List[(port, pid, name, status, cmdline)]
    """
    results: List[Tuple[int, int, str, str, str]] = []
    seen = set()  # (port, pid) 去重
    
    if shutil.which("lsof") is None:
        return results
    
    rc, out = _run_command(["lsof", "-nP", "-iTCP", "-iUDP"])
    if rc != 0:
        return results
    
    # 解析 lsof 输出
    # 格式: COMMAND PID USER FD TYPE DEVICE SIZE/OFF NODE NAME
    # 示例: nginx 1234 user 6u IPv4 0x... TCP *:8080 (LISTEN)
    lines = [l for l in out.splitlines() if l.strip()]
    if not lines:
        return results
    
    for line in lines[1:]:  # 跳过表头
        parts = re.split(r"\s+", line)
        if len(parts) < 9:
            continue
        
        name = parts[0]
        try:
            pid = int(parts[1])
        except ValueError:
            continue
        
        # 提取端口和状态
        name_part = ' '.join(parts[8:])
        port_match = re.search(r":(\d+)", name_part)
        if not port_match:
            continue
        port = int(port_match.group(1))
        
        status = ""
        if "LISTEN" in name_part:
            status = "LISTEN"
        elif "ESTABLISHED" in name_part:
            status = "ESTABLISHED"
        
        key = (port, pid)
        if key in seen:
            continue
        seen.add(key)
        
        # 先收集所有 PID，稍后批量获取命令行
        results.append((port, pid, name, status, ""))  # 临时空命令行
    
    # 批量获取所有进程的命令行（优化：一次性获取）
    if results:
        all_pids = {pid for port, pid, name, status, cmdline in results}
        pid_cmdline_map = _get_all_process_info_mac()
        
        # 更新结果中的命令行
        final_results = []
        for port, pid, name, status, _ in results:
            cmdline = pid_cmdline_map.get(pid, "")
            final_results.append((port, pid, name, status, cmdline))
        return final_results
    
    return results


def get_all_ports() -> List[Tuple[int, int, str, str, str]]:
    """获取所有端口使用情况（供 whoport 等模块复用）
    
    返回: List[(port, pid, name, status, cmdline)]
    """
    results = _try_psutil()
    if not results:
        system = platform.system().lower()
        if system == 'windows':
            results = _windows_list()
        elif system == 'linux':
            results = _linux_list()
        elif system == 'darwin':
            results = _mac_list()
        else:
            results = []
    results.sort(key=lambda x: x[0])
    return results


def command_port_list(full: bool = False) -> None:
    """列出所有端口使用情况
    
    Args:
        full: 如果为 True，显示完整的命令行（不截断）
    """
    import sys
    
    results = get_all_ports()
    
    if not results:
        print("未发现端口使用情况或缺少权限/工具无法识别。")
        print("建议：以管理员/root 权限运行，或安装 psutil。")
        return
    
    # 处理 Windows 控制台编码问题 - 使用安全的输出函数
    def safe_print(text: str) -> None:
        """安全打印，处理编码问题"""
        try:
            print(text)
        except UnicodeEncodeError:
            # 如果遇到编码问题，使用 errors='replace' 重新编码
            try:
                if sys.stdout.encoding:
                    encoded = text.encode(sys.stdout.encoding, errors='replace')
                    sys.stdout.buffer.write(encoded)
                    sys.stdout.buffer.write(b'\n')
                else:
                    # 备用方案：使用 ASCII
                    safe_text = text.encode('ascii', 'replace').decode('ascii')
                    print(safe_text)
            except Exception:
                # 最后的备用方案
                print(text.encode('ascii', 'replace').decode('ascii', 'replace'))
    
    # 打印表头
    try:
        safe_print(f"{'端口':<8} {'PID':<8} {'进程名':<20} {'状态':<12} {'命令行'}")
    except Exception:
        safe_print(f"{'Port':<8} {'PID':<8} {'Process':<20} {'Status':<12} {'Command'}")
    safe_print("-" * 150)
    
    # 打印结果
    for port, pid, name, status, cmdline in results:
        # 根据 full 参数决定是否截断命令行
        if not full:
            # 默认截断到150字符，足够显示大部分完整路径
            max_cmdline_len = 150
            if len(cmdline) > max_cmdline_len:
                cmdline = cmdline[:max_cmdline_len - 3] + "..."
        # 如果 full=True，显示完整命令行，不截断
        status_str = status if status else "-"
        safe_print(f"{port:<8} {pid:<8} {name:<20} {status_str:<12} {cmdline}")




#!/usr/bin/env python3
import os
import subprocess
import sys


def find_wpsmain_root(current_path: str) -> str:
    """查找 wpsmain 根目录"""
    path = os.path.abspath(current_path)
    
    # 从当前路径开始向上查找，直到找到包含 wpsmain 的目录
    while path != os.path.dirname(path):
        wpsmain_path = os.path.join(path, "wpsmain")
        if os.path.isdir(wpsmain_path):
            return wpsmain_path
        path = os.path.dirname(path)
    
    # 如果当前目录就是 wpsmain
    if os.path.basename(current_path) == "wpsmain":
        return os.path.abspath(current_path)
    
    return ""


def run_docker_command(command: str, cwd: str) -> bool:
    """在 Docker 中执行命令"""
    docker_cmd = ["kdocker", "-r", "qt5"] + command.split()
    
    print(f"执行命令: {' '.join(docker_cmd)}")
    print(f"工作目录: {cwd}")
    
    try:
        # 实时输出到终端
        process = subprocess.Popen(
            docker_cmd,
            cwd=cwd,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,
            encoding='utf-8',
            errors='replace',
            bufsize=1,
            universal_newlines=True
        )
        
        # 实时读取并打印输出
        while True:
            output = process.stdout.readline()
            if output == '' and process.poll() is not None:
                break
            if output:
                print(output.rstrip())
        
        # 等待进程完成并获取返回码
        return_code = process.poll()
        
        if return_code == 0:
            return True
        else:
            print(f"命令执行失败，返回码: {return_code}", file=sys.stderr)
            return False
        
    except Exception as e:
        print(f"命令执行失败: {e}", file=sys.stderr)
        return False


def check_debug_weboffice_exists(wpsmain_path: str) -> bool:
    """检查 debug_weboffice 目录是否存在"""
    debug_path = os.path.join(wpsmain_path, "debug_weboffice")
    return os.path.isdir(debug_path)


def find_wpsweb_build_script(current_path: str) -> str:
    """查找 wpsweb 目录中的 build_server.sh 脚本"""
    path = os.path.abspath(current_path)
    
    # 从当前路径开始向上查找，直到找到包含 wpsweb 的目录
    while path != os.path.dirname(path):
        wpsweb_path = os.path.join(path, "wpsweb")
        if os.path.isdir(wpsweb_path):
            build_script_path = os.path.join(wpsweb_path, "build_server.sh")
            if os.path.isfile(build_script_path) and os.access(build_script_path, os.X_OK):
                return build_script_path
        path = os.path.dirname(path)
    
    # 如果当前目录就是 wpsweb
    if os.path.basename(current_path) == "wpsweb":
        build_script_path = os.path.join(current_path, "build_server.sh")
        if os.path.isfile(build_script_path) and os.access(build_script_path, os.X_OK):
            return build_script_path
    
    return ""


def command_auto_build() -> None:
    """自动编译 wpsmain 命令"""
    # 获取当前工作目录
    current_dir = os.getcwd()
    
    # 查找 wpsmain 根目录
    wpsmain_root = find_wpsmain_root(current_dir)
    
    if not wpsmain_root:
        print("错误: 未找到 wpsmain 目录", file=sys.stderr)
        print("请确保当前目录在 wpsmain 或其父级目录中", file=sys.stderr)
        sys.exit(1)
    
    print(f"找到 wpsmain 目录: {wpsmain_root}")
    
    # 步骤 1: 在 wpsmain 目录中执行 krepo-ng config --new -x weboffice
    print("\n=== 步骤 1: 配置 weboffice ===")
    if not run_docker_command("krepo-ng config --new -x weboffice", wpsmain_root):
        print("配置失败", file=sys.stderr)
        sys.exit(1)
    
    # 步骤 2: 检查是否创建了 debug_weboffice 目录
    print("\n=== 步骤 2: 检查 debug_weboffice 目录 ===")
    if not check_debug_weboffice_exists(current_dir):
        print("错误: 未找到 debug_weboffice 目录", file=sys.stderr)
        print("配置可能失败，请检查 krepo-ng config 命令的输出", file=sys.stderr)
        sys.exit(1)
    
    debug_weboffice_path = os.path.join(current_dir, "debug_weboffice")
    print(f"找到 debug_weboffice 目录: {debug_weboffice_path}")
    
    # 步骤 3: 进入 debug_weboffice 目录执行 krepo-ng build
    print("\n=== 步骤 3: 编译 weboffice ===")
    if not run_docker_command("krepo-ng build", debug_weboffice_path):
        print("编译失败", file=sys.stderr)
        sys.exit(1)
    
    # 步骤 4: 查找并执行 wpsweb 中的 build_server.sh 脚本
    print("\n=== 步骤 4: 执行 wpsweb build_server.sh ===")
    build_script_path = find_wpsweb_build_script(current_dir)
    
    if not build_script_path:
        print("警告: 未找到可执行的 build_server.sh 脚本", file=sys.stderr)
        print("请确保 wpsweb 目录中存在 build_server.sh 文件且具有执行权限", file=sys.stderr)
    else:
        print(f"找到 build_server.sh 脚本: {build_script_path}")
        wpsweb_dir = os.path.dirname(build_script_path)
        
        # 在 Docker 中执行 build_server.sh 脚本
        if not run_docker_command(f"./build_server.sh", wpsweb_dir):
            print("build_server.sh 执行失败", file=sys.stderr)
            sys.exit(1)
    
    print("\n=== 编译完成 ===")
    print("wpsmain 自动编译流程已完成")
    print("完成。") 
    